package net.npaka.wifidirectex;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import android.content.Context;
import android.os.Handler;

//チャットマネージャ
public class ChatManager {
    //状態定数
    public static final int STATE_NONE      =0;
    public static final int STATE_LISTEN    =1;
    public static final int STATE_CONNECTING=2;
    public static final int STATE_CONNECTED =3;

    //変数
    private int             state;     //状態
    private Handler         handler;   //ハンドラ
    private AcceptThread    acceptT;   //サーバの接続待ちスレッド
    private ConnectThread   connectT;  //クライアントの接続要求スレッド
    private ConnectedThread connectedT;//接続中の処理スレッド
    
    //コンストラクタ
    public ChatManager(Context context,Handler handler) {
        this.state  =STATE_NONE;
        this.handler=handler;
    }

    //状態の指定
    private synchronized void setState(int state) {
        this.state=state;
        handler.obtainMessage(
            WiFiDirectEx.MSG_STATE_CHANGE,state,-1).sendToTarget();
    }

    //状態の取得
    public synchronized int getState() {
        return state;
    }

    //サーバの接続待ちスレッドの開始
    public synchronized void start() {
        if (acceptT==null) {
            acceptT=new AcceptThread();
            acceptT.start();
        }
    }    
    
    //クライアントの接続要求スレッドの開始
    public synchronized void connect(String host) {
        if (state==STATE_CONNECTING) {
            if (connectT!=null) {connectT.cancel();connectT=null;}
        }
        if (connectedT!=null) {connectedT.cancel();connectedT=null;}
        connectT=new ConnectThread(host);
        connectT.start();
        setState(STATE_CONNECTING);
    }    
    
    //接続中の処理スレッドの開始
    public synchronized void connected(Socket socket) {
        if (connectedT!=null) {connectedT.cancel();connectedT=null;}
        connectedT=new ConnectedThread(socket);
        connectedT.start();
        setState(STATE_CONNECTED);
    }

    //送信データの書き込み
    public synchronized void write(byte[] out) {
        if (state!=STATE_CONNECTED) return;
        connectedT.write(out);
    }
    
    //スレッドの停止
    public synchronized void stop() {
        if (connectT  !=null) {connectT.cancel();  connectT  =null;}
        if (connectedT!=null) {connectedT.cancel();connectedT=null;}
        setState(STATE_NONE);
    }

    //サーバの接続待ちスレッド(10)
    private class AcceptThread extends Thread {
        private ServerSocket serverSocket;
        
        //処理
        public void run() {
            Socket socket=null;
            try {
                serverSocket=new ServerSocket(8988);
                while (acceptT!=null) {
                    socket=serverSocket.accept();
                    connected(socket);
                }
            } catch (Exception e) {
                android.util.Log.e("debug",e.toString());
            }
        }
    }
    
    //クライアントの接続要求スレッド(11)
    private class ConnectThread extends Thread {
        private String host;
        private Socket socket;

        //コンストラクタ
        public ConnectThread(String host) {
            this.host=host;
        }

        //処理
        public void run() {
            try {
                socket=new Socket(host,8988);
                connectT=null;
                connected(socket);
            } catch (Exception e) {
                android.util.Log.e("debug",e.toString());
                setState(STATE_LISTEN);
                try {
                    socket.close();
                } catch (Exception e2) {
                }
            }
        }

        //キャンセル
        public void cancel() {
            try {
                socket.close();
            } catch (Exception e) {
            }
        }
    }    
    
    //接続中の処理スレッド(12)
    private class ConnectedThread extends Thread {
        private Socket       socket;
        private InputStream  in;
        private OutputStream out;

        //コンストラクタ
        public ConnectedThread(Socket socket) {
            try {
                this.socket=socket;
                this.in    =socket.getInputStream();
                this.out   =socket.getOutputStream();
            } catch (Exception e) {
                android.util.Log.e("debug",e.toString());
            }
        }

        //処理
        public void run() {
            byte[] buf=new byte[1024];
            while (true) {
                try {
                    int size=in.read(buf);
                    if (size<=0) throw new Exception();
                    handler.obtainMessage(WiFiDirectEx.MSG_READ,
                        size,-1,buf).sendToTarget();
                } catch (Exception e) {
                    android.util.Log.e("debug",e.toString());
                    setState(STATE_LISTEN);
                    break;
                }
            }
        }

        //書き込み
        public void write(byte[] buf) {
            try {
                out.write(buf);
            } catch (Exception e) {
            }
        }

        //キャンセル
        public void cancel() {
            try {
                socket.close();
            } catch (Exception e) {
            }
        }
    }
}
