package net.npaka.wifidirectex;
import java.util.ArrayList;
import java.util.List;
import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Color;
import android.net.wifi.WpsInfo;
import android.net.wifi.p2p.WifiP2pConfig;
import android.net.wifi.p2p.WifiP2pDevice;
import android.net.wifi.p2p.WifiP2pDeviceList;
import android.net.wifi.p2p.WifiP2pInfo;
import android.net.wifi.p2p.WifiP2pManager;
import android.net.wifi.p2p.WifiP2pManager.ActionListener;
import android.net.wifi.p2p.WifiP2pManager.Channel;
import android.net.wifi.p2p.WifiP2pManager.ConnectionInfoListener;
import android.net.wifi.p2p.WifiP2pManager.PeerListListener;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.provider.Settings;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.TextView;

//Wi-Fi Direct
public class WiFiDirectEx extends Activity implements 
    View.OnClickListener,PeerListListener,ConnectionInfoListener {
    private final static String BR=System.getProperty("line.separator");    
    private final static int WC=LinearLayout.LayoutParams.WRAP_CONTENT;
    private final static int MP=LinearLayout.LayoutParams.MATCH_PARENT;

    //メッセージ定数
    public static final int MSG_STATE_CHANGE=1;
    public static final int MSG_READ        =2;
    
    //リクエスト定数
    private static final int RQ_CONNECT_DEVICE=1;
    
    //Wi-Fi Direct
    private WifiP2pManager      manager;    //Wi-Fi P2Pマネージャ
    private Channel             channel;    //チャンネル
    private BroadcastReceiver   receiver;   //ブロードキャストレシーバー
    private boolean             enabled;    //Wi-Fi Directの有効・無効
    private String              status="";  //ステータス
    private ChatManager         chatManager;//チャットマネージャ
    private List<WifiP2pDevice> devices=new ArrayList<WifiP2pDevice>();
    
    //UI
    private EditText edtSend;   //送信エディットテキスト
    private Button   btnSend;   //送信ボタン
    private TextView lblReceive;//受信ラベル
    
    //アクティビティ起動時に呼ばれる
    @Override
    public void onCreate(Bundle bundle) {
        super.onCreate(bundle);
        //レイアウトの生成
        LinearLayout layout=new LinearLayout(this);
        layout.setOrientation(LinearLayout.VERTICAL);
        setContentView(layout); 
        setTitle("WiFiDirectEx ["+(enabled?"ON":"OFF")+"/"+status+"]");
        
        //送信エディットテキストの生成
        edtSend=new EditText(this);
        edtSend.setId(2);
        edtSend.setText("",TextView.BufferType.NORMAL);
        edtSend.setLayoutParams(new LinearLayout.LayoutParams(MP,WC));
        edtSend.setEnabled(false);
        layout.addView(edtSend);
        
        //送信ボタンの生成
        btnSend=new Button(this);
        btnSend.setText("送信");
        btnSend.setOnClickListener(this);
        btnSend.setLayoutParams(new LinearLayout.LayoutParams(WC,WC));
        btnSend.setEnabled(false);
        layout.addView(btnSend);

        //受信ラベルの生成
        lblReceive=new TextView(this);
        lblReceive.setId(1);
        lblReceive.setText("");
        lblReceive.setTextSize(16.0f);
        lblReceive.setTextColor(Color.rgb(255,255,255));
        lblReceive.setLayoutParams(new LinearLayout.LayoutParams(MP,WC));
        layout.addView(lblReceive);   
        
        //Wi-Fi P2Pマネージャの初期化(1)
        manager=(WifiP2pManager)getSystemService(Context.WIFI_P2P_SERVICE);
        channel=manager.initialize(this,getMainLooper(),null);   
        
        //チャットマネージャ
        chatManager=new ChatManager(this,handler);
    }
    
    //アクティビティスタート時に呼ばれる
    @Override
    public void onStart() {
        super.onStart();
        
        //Wi-Fi Directのブロードキャストレシーバーの登録(2)
        receiver=new WiFiDirectReceiver(manager,channel,this);
        IntentFilter filter=new IntentFilter();
        filter.addAction(WifiP2pManager.WIFI_P2P_STATE_CHANGED_ACTION);
        filter.addAction(WifiP2pManager.WIFI_P2P_PEERS_CHANGED_ACTION);
        filter.addAction(WifiP2pManager.WIFI_P2P_CONNECTION_CHANGED_ACTION);
        filter.addAction(WifiP2pManager.WIFI_P2P_THIS_DEVICE_CHANGED_ACTION);
        registerReceiver(receiver,filter);
    }
    
    //アクティビティ再開時に呼ばれる
    @Override
    public synchronized void onResume() {
        super.onResume();
        
        //サーバの接続待ちスレッドの開始
        chatManager.start();
    }
    
    //アクティビティ停止時に呼ばれる
    @Override
    public void onStop() {
        super.onStop();
        
        //スレッドの停止
        chatManager.stop();
        
        //端末との切断
        disconnectDevice();
    }
    
    //アクティビティ破棄時に呼ばれる
    @Override
    public void onDestroy() {
        super.onDestroy();
        
        //Wi-Fi Directのブロードキャストレシーバーの解除(2)
        unregisterReceiver(receiver);
    }
    
    //通信の停止
    public void stop() {
        chatManager.stop();
    }
    
    //チャットサーバから情報を取得するハンドラ
    private final Handler handler=new Handler() {
        @Override
        public void handleMessage(Message msg) {
            //状態変更
            if (msg.what==MSG_STATE_CHANGE) {
                if (msg.arg1==ChatManager.STATE_CONNECTED) {
                    edtSend.setEnabled(true);
                    btnSend.setEnabled(true);
                    addText("ソケット接続完了");
                } else {
                    edtSend.setEnabled(false);
                    btnSend.setEnabled(false);
                    if (msg.arg1==ChatManager.STATE_CONNECTING) {
                        addText("ソケット接続中");
                    } else {
                        addText("ソケット未接続");
                    }
                }
            }
            //メッセージ受信
            else if (msg.what==MSG_READ && msg.arg1>0){
                addText(new String((byte[])msg.obj,0,msg.arg1));
            }
        }
    };    
    
    //端末本体のWi-Fi Directの有効・無効の指定(3)
    public void setEnabled(boolean enabled) {
        this.enabled=enabled;
        setTitle("WiFiDirectEx ["+(enabled?"ON":"OFF")+"/"+status+"]");
    }
    
    //Wi-Fi Direct端末一覧の更新時に呼ばれる(4)
    public void onPeersAvailable(WifiP2pDeviceList deviceList) {
        devices.clear();
        devices.addAll(deviceList.getDeviceList());
        if (DeviceListActivity.activity!=null) {
            DeviceListActivity.activity.update(devices);
        }
    }
    
    //Wi-Fi Direct接続情報の通知時に呼ばれる(5)
    public void onConnectionInfoAvailable(WifiP2pInfo info) {
        if (chatManager.getState()==ChatManager.STATE_NONE && !info.isGroupOwner) {
            addText(info.groupOwnerAddress.getHostAddress());
            chatManager.connect(info.groupOwnerAddress.getHostAddress());
        }
    }
    
    //端末本体のステータスの指定(6)
    public void setStatus(String status) {
        this.status=status;
        setTitle("WiFiDirectEx ["+(enabled?"ON":"OFF")+"/"+status+"]");
    }
    
    //端末との接続(7)
    public void connectDevice(final WifiP2pDevice device) {
        //接続設定の生成
        WifiP2pConfig config=new WifiP2pConfig();
        config.deviceAddress=device.deviceAddress;
        config.wps.setup=WpsInfo.PBC;
        manager.connect(channel,config,new ActionListener() {
            //接続成功時に呼ばれる
            public void onSuccess() {
                addText("端末との接続成功");
            }
 
            //接続失敗時に呼ばれる
            public void onFailure(int reason) {
                addText("端末との接続失敗");
            }
        });
    }
    
    //端末との切断(8)
    public void disconnectDevice() {
        manager.removeGroup(channel, new ActionListener() {
            //成功時に呼ばれる
            public void onSuccess() {
                addText("端末との切断成功");
            }
            
            //失敗時に呼ばれる
            public void onFailure(int reasonCode) {
                addText("端末との切断失敗");
             }
        });
    }
    
    //オプションメニュー生成時に呼ばれる
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        MenuItem item0=menu.add(0,0,0,"端末検索");
        item0.setIcon(android.R.drawable.ic_search_category_default);
        MenuItem item1=menu.add(0,1,0,"通信設定");
        item1.setIcon(android.R.drawable.ic_menu_preferences);
        return true;
    }

    //オプションメニュー選択時に呼ばれる
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId()==0) {
            //端末検索(6)
            Intent intent=new Intent(this,DeviceListActivity.class);
            startActivityForResult(intent,RQ_CONNECT_DEVICE);
            manager.discoverPeers(channel,new WifiP2pManager.ActionListener() {
                @Override
                public void onSuccess() {
                    addText("端末の検索成功");
                }

                @Override
                public void onFailure(int reasonCode) {
                    addText("端末の検索失敗");
                }
            });            
            return true;
        } else if (item.getItemId()==1) {
            //通信設定
            startActivity(new Intent(Settings.ACTION_WIRELESS_SETTINGS));
            return true;
        }
        return false;
    }    
    
    //アクティビティ復帰時に呼ばれる
    public void onActivityResult(int requestCode,int resultCode,Intent data) {
        //端末検索
        if (requestCode==RQ_CONNECT_DEVICE ){
            if (resultCode==Activity.RESULT_OK) {
                //端末との接続
                int idx=data.getExtras().getInt("device_index");
                connectDevice(devices.get(idx));
            }
        }
    }
    
    //受信テキストの追加
    private void addText(final String text) {
        lblReceive.setText(text+BR+lblReceive.getText());
    }
    
    //ボタンクリックイベントの処理
    public void onClick(View v) {
        try {
            //メッセージの送信
            String message=edtSend.getText().toString();
            if (message.length()<=0) return;
            chatManager.write(message.getBytes());
            addText(message);
            edtSend.setText("",TextView.BufferType.NORMAL);
        } catch (Exception e) {
            addText("送信失敗");
        }           
    }  
}
