package net.npaka.wifidirectex;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.net.NetworkInfo;
import android.net.wifi.p2p.WifiP2pDevice;
import android.net.wifi.p2p.WifiP2pManager;
import android.net.wifi.p2p.WifiP2pManager.Channel;

//Wi-Fi Directのブロードキャストレシーバー
public class WiFiDirectReceiver extends BroadcastReceiver {
    private WifiP2pManager manager;
    private Channel        channel;
    private WiFiDirectEx   activity;

    //コンストラクタ
    public WiFiDirectReceiver(WifiP2pManager manager,
        Channel channel,WiFiDirectEx activity) {
        this.manager =manager;
        this.channel =channel;
        this.activity=activity;
    }

    //受信時に呼ばれる
    @Override
    public void onReceive(Context context,Intent intent) {
        String action=intent.getAction();
        //端末本体のWi-Fi Directの有効・無効の変更通知(3)
        if (WifiP2pManager.WIFI_P2P_STATE_CHANGED_ACTION.equals(action)) {
            int state=intent.getIntExtra(WifiP2pManager.EXTRA_WIFI_STATE,-1);
            if (state==WifiP2pManager.WIFI_P2P_STATE_ENABLED) {
                activity.setEnabled(true);
            } else {
                activity.setEnabled(false);
            }
        } 
        //Wi-Fi Direct端末一覧の変更通知(4)
        else if (WifiP2pManager.WIFI_P2P_PEERS_CHANGED_ACTION.equals(action)) {
            if (manager==null) return;
            manager.requestPeers(channel,activity);
        } 
        //Wi-Fi Direct通信状態の変更通知(5)
        else if (WifiP2pManager.WIFI_P2P_CONNECTION_CHANGED_ACTION.equals(action)) {
            if (manager==null) return;
            NetworkInfo networkInfo=(NetworkInfo)intent.
                getParcelableExtra(WifiP2pManager.EXTRA_NETWORK_INFO);
            
            //接続
            if (networkInfo.isConnected()) {
                manager.requestConnectionInfo(channel,activity);
            } else {
                activity.stop();
            }
        } 
        //端末本体の状態の変更通知(6)
        else if (WifiP2pManager.WIFI_P2P_THIS_DEVICE_CHANGED_ACTION.equals(action)) {
            WifiP2pDevice device=(WifiP2pDevice)intent.getParcelableExtra(
                WifiP2pManager.EXTRA_WIFI_P2P_DEVICE);
            String status="";
            if (device.status==WifiP2pDevice.AVAILABLE)   status="available";
            if (device.status==WifiP2pDevice.INVITED)     status="invited";
            if (device.status==WifiP2pDevice.CONNECTED)   status="connected";
            if (device.status==WifiP2pDevice.FAILED)      status="failed";
            if (device.status==WifiP2pDevice.UNAVAILABLE) status="unavailable";
            activity.setStatus(status);
        }
    }
}
