package net.npaka.dbproviderex;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.graphics.Color;
import android.net.Uri;
import android.os.Bundle;
import android.view.View;
import android.view.Window;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;

//DBを提供するコンテンツプロバイダ
public class DBProviderEx extends Activity 
    implements View.OnClickListener {
    private final static int WC=LinearLayout.LayoutParams.WRAP_CONTENT;
    private final static int MP=LinearLayout.LayoutParams.MATCH_PARENT;
    private EditText editText;
    
    //アクティビティ起動時に呼ばれる
    @Override
    public void onCreate(Bundle bundle) {
        super.onCreate(bundle);
        requestWindowFeature(Window.FEATURE_NO_TITLE);

        //レイアウトの生成
        LinearLayout layout=new LinearLayout(this);
        layout.setBackgroundColor(Color.rgb(255,255,255));
        layout.setOrientation(LinearLayout.VERTICAL);
        setContentView(layout);            
        
        //エディットテキストの生成
        editText=new EditText(this);
        editText.setLayoutParams(new LinearLayout.LayoutParams(MP,WC));
        layout.addView(editText);

        //ボタンの生成
        layout.addView(makeButton("書き込み","write"));        
        layout.addView(makeButton("読み込み","read"));        
    }
    
    //ボタンの生成
    private Button makeButton(String text,String tag) {
        Button button=new Button(this);
        button.setText(text);
        button.setTag(tag);
        button.setOnClickListener(this);
        button.setLayoutParams(new LinearLayout.LayoutParams(WC,WC));
        return button;
    }    

    //ボタンクリックイベントの処理
    public void onClick(View v) {
        String tag=(String)v.getTag();
        if (tag.equals("write")) {
            try {
                writeDB(editText.getText().toString());
            } catch (Exception e) {
                showDialog(this,"エラー","書き込み失敗しました ");
            }            
        } else if (tag.equals("read")) {
            try {
                editText.setText(readDB());               
            } catch (Exception e) {
                showDialog(this,"エラー","読み込み失敗しました "+e.toString());
            }            
        }
    }    
    
    //DBへの書き込み
    private void writeDB(String info) throws Exception {
        //コンテンツプロバイダが提供するデータベースを示すURI(1)
        Uri uri=Uri.parse("content://net.npaka.dbprovider/"); 
       
        //コンテンツプロバイダが提供するデータベースへのアクセス(2)
        ContentValues values=new ContentValues();
        values.put("id","0");
        values.put("info",info);
        int num=getContentResolver().update(uri,values,null,null);
        if (num==0) getContentResolver().insert(uri,values);
    }

    //DBからの読み込み
    private String readDB() throws Exception {
        //コンテンツプロバイダが提供するデータベースを示すURI(1)
        Uri uri=Uri.parse("content://net.npaka.dbprovider/"); 
        
        //コンテンツプロバイダが提供するデータベースへのアクセス(2)
        Cursor c=this.getContentResolver().query(uri,
            new String[]{"id","info"},"id='0'",null,null);
        if (c.getCount()==0) throw new Exception();
        c.moveToFirst();
        String str=c.getString(1);
        c.close();
        return str;
    }
    
    //ダイアログの表示
    private static void showDialog(Context context,String title,String text) {
        AlertDialog.Builder ad=new AlertDialog.Builder(context);
        ad.setTitle(title);
        ad.setMessage(text);
        ad.setPositiveButton("OK",null);
        ad.show();
    } 
}