package net.npaka.twitterclient;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.os.Bundle;
import android.os.Handler;
import android.util.DisplayMetrics;
import android.util.Xml;
import android.view.Gravity;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
import oauth.signpost.OAuthProvider;
import oauth.signpost.basic.DefaultOAuthProvider;
import oauth.signpost.commonshttp.CommonsHttpOAuthConsumer;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.xmlpull.v1.XmlPullParser;

//Twitterクライアント
public class TwitterClient extends Activity {
    //定数
    private final static int WC=LinearLayout.LayoutParams.WRAP_CONTENT;
    private final static int MP=LinearLayout.LayoutParams.MATCH_PARENT;
    private final static String BR=System.getProperty("line.separator");
    private static final int MENU_SETUP =0;
    private static final int MENU_UPDATE=1;
    
    //Twitter定数
    private final static String CONSUMER_KEY   ="コンシューマーキーを記述";
    private final static String CONSUMER_SECRET="コンシューマーシークレットを記述";
    private final String CALLBACKURL="myapp://mainactivity";

    //認証
    private CommonsHttpOAuthConsumer consumer;
    private OAuthProvider            provider;
    private String error;
    
    //UI
    private ProgressDialog progressDlg;
    private ListView listView;
    private ArrayList<Status> timeline=new ArrayList<Status>();
    private HashMap<String,Drawable> icons=new HashMap<String,Drawable>();
    private Handler handler=new Handler();
    private float dpi;

    //アプリの初期化
    @Override
    public void onCreate(Bundle bundle) {
        super.onCreate(bundle);
        
        //レイアウトの生成
        LinearLayout layout=new LinearLayout(this);
        layout.setOrientation(LinearLayout.VERTICAL);
        setContentView(layout); 
        
        //DPIの取得(9)
        DisplayMetrics metrics=new DisplayMetrics();  
        getWindowManager().getDefaultDisplay().getMetrics(metrics); 
        dpi=metrics.density;

        //リストビューの生成
        listView=new ListView(this);
        listView.setLayoutParams(new LinearLayout.LayoutParams(MP,WC));
        listView.setAdapter(new TwitterAdapter());
        layout.addView(listView);
        
        //プログレスダイアログの生成(4)
        progressDlg=new ProgressDialog(this);
        progressDlg.setCancelable(false);
        progressDlg.setMessage("通信中…");
        progressDlg.setProgressStyle(ProgressDialog.STYLE_SPINNER);
        
        //OAuth認証による通信
        doOauth(false);
    }

    //オプションメニューの生成
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        MenuItem item0=menu.add(0,MENU_SETUP,0,"設定");
        item0.setIcon(android.R.drawable.ic_menu_preferences);
        MenuItem item1=menu.add(0,MENU_UPDATE,0,"更新");
        item1.setIcon(android.R.drawable.ic_menu_upload);
        return true;
    }    

    //メニューアイテム選択イベントの処理
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int itemId=item.getItemId();
        
        //設定
        if (itemId==MENU_SETUP) {
            doOauth(true);
        }
        //更新
        else if (itemId==MENU_UPDATE) {
            if (timeline.size()==0) {
                doOauth(false);
            } else {
                updateTimeline();
            }
        }
        return true;
    }
    
    //OAuth認証による通信(1)
    private void doOauth(boolean setup) {
        try {
            //コンシューマとプロバイダの生成
            consumer=new CommonsHttpOAuthConsumer(
                CONSUMER_KEY,CONSUMER_SECRET);
            provider=new DefaultOAuthProvider(
                "https://api.twitter.com/oauth/request_token",
                "https://api.twitter.com/oauth/access_token",
                "https://api.twitter.com/oauth/authorize");
            
            //トークンの読み込み
            SharedPreferences pref=getSharedPreferences(
                "token",MODE_PRIVATE);
            String token      =pref.getString("token","");
            String tokenSecret=pref.getString("tokenSecret","");
            
            //認証済みかつ設定以外の時
            if (!setup && token.length()>0 && tokenSecret.length()>0) {
                //コンシューマのトークン指定
                consumer.setTokenWithSecret(token,tokenSecret);
                
                //タイムラインの更新
                updateTimeline();
            } 
            //未認証または設定の時
            else {
                //ブラウザで認証サイトを開く
                openOauthSite();
            }
        } catch (Exception e) {
            toast(e.getMessage());
        }
    }
        
    //ブラウザで認証サイトを開く(2)
    private void openOauthSite() {
        //スレッドの生成
        Thread thread=new Thread(){public void run(){
            try {
                //認証サイトのURLの取得
                final String url=provider.retrieveRequestToken(
                    consumer,CALLBACKURL);
                
                //ハンドラの生成
                handler.post(new Runnable() {public void run(){
                    //ブラウザで認証サイトを開く
                    TwitterClient.this.startActivity(new Intent(
                        Intent.ACTION_VIEW,Uri.parse(url)));
                }});
            } catch (Exception e) {
                toast(e.getMessage());
            }
        }};
        thread.start();
    }
    
    //インテントを受信した時に呼ばれる
    @Override
    protected void onNewIntent(Intent intent) {
        super.onNewIntent(intent);
        //認証完了時の処理(3)
        final Uri uri=intent.getData();
        if (uri!=null && uri.toString().startsWith(CALLBACKURL)) {
            //スレッドの生成
            Thread thread=new Thread(new Runnable(){public void run(){
            try {
                //認証情報の取得
                String verifier=uri.getQueryParameter(
                    oauth.signpost.OAuth.OAUTH_VERIFIER);
                provider.retrieveAccessToken(consumer,verifier);

                //トークンの書き込み
                SharedPreferences pref=getSharedPreferences(
                    "token",MODE_PRIVATE);
                SharedPreferences.Editor editor=pref.edit();
                editor.putString("token",consumer.getToken());
                editor.putString("tokenSecret",consumer.getTokenSecret());
                editor.commit();

                //ハンドラの生成
                handler.post(new Runnable(){public void run() {
                    //タイムラインの更新
                    updateTimeline();
                }});
            } catch(Exception e){
                toast(e.getMessage());
            }
            }});
            thread.start();
        }
    }    
    
    //タイムラインの更新(5)
    private void updateTimeline() {
        //プログレスダイアログの表示
        progressDlg.show();//(4)
        
        //スレッドの生成
        Thread thread=new Thread(new Runnable(){public void run(){
            error=null;
            final ArrayList<Status> timelineBuf=new ArrayList<Status>();
            Status status =null;
            String tagName=null;
            InputStream in=null;
            try {
                //証明書付きのHTTP通信(6)
                DefaultHttpClient http=new DefaultHttpClient();
                HttpGet httpGet=new HttpGet(
                    "http://twitter.com/statuses/friends_timeline.xml"); 
                consumer.sign(httpGet);
                HttpResponse execute=http.execute(httpGet);
                in=execute.getEntity().getContent();

                //XMLのパース処理(7)
                XmlPullParser parser=Xml.newPullParser();
                parser.setInput(new InputStreamReader(in));
                while (true) {
                    int type=parser.next();
                    //ドキュメント開始
                    if (type==XmlPullParser.START_DOCUMENT) {
                    } 
                    //タグ開始
                    else if (type==XmlPullParser.START_TAG) {
                        tagName=parser.getName();
                        if (tagName.equals("status")) {
                            status=new Status();
                            timelineBuf.add(status);
                        }
                    } 
                    //テキスト
                    else if (type==XmlPullParser.TEXT) {
                        if (parser.getText().trim().length()==0) {
                        } else if (tagName.equals("screen_name")) {
                            status.name=parser.getText();
                        } else if (tagName.equals("text")) {
                            status.text=parser.getText();
                        } else if (tagName.equals("profile_image_url")) {
                            status.iconURL=parser.getText();
                        }
                    } 
                    //タグ終了
                    else if (type==XmlPullParser.END_TAG) {
                    } 
                    //ドキュメント終了
                    else if (type==XmlPullParser.END_DOCUMENT) {
                        break;
                    }                
                }
                in.close();

                //アイコンの読み込み
                for (int i=0;i<timelineBuf.size();i++) {
                    timelineBuf.get(i).icon=
                        readIcon(timelineBuf.get(i).iconURL);
                }
            } catch (Exception e) {
                error=e.getMessage();
            }
            
            //ハンドラの生成
            handler.post(new Runnable(){public void run(){
                //プログレスダイアログの非表示(4)
                progressDlg.dismiss();
                
                //リストビューの更新
                if (error==null) {
                    timeline=timelineBuf;
                    ((BaseAdapter)listView.getAdapter()).notifyDataSetChanged();
                } else {
                    toast(error);
                }
            }});
        }});
        thread.start();
    }
    
    //アイコンの読み込み
    private Drawable readIcon(String url) throws Exception {
        Drawable drawable=icons.get(url);
        if (drawable!=null) return drawable;
        byte[] data=http2data(url);
        Bitmap bmp=BitmapFactory.decodeByteArray(data,0,data.length);
        drawable=new BitmapDrawable(bmp);
        icons.put(url,drawable);
        return drawable;
    }

    //HTTP通信
    private byte[] http2data(String path) throws Exception {
        int size;
        byte[] w=new byte[1024]; 
        HttpURLConnection c=null;
        InputStream in=null;
        ByteArrayOutputStream out=null;
        try {
            //HTTP接続のオープン
            URL url=new URL(path);
            c=(HttpURLConnection)url.openConnection();
            c.setRequestMethod("GET");
            c.connect();
            in=c.getInputStream();
            
            //バイト配列の読み込み
            out=new ByteArrayOutputStream();
            while (true) {
                size=in.read(w);
                if (size<=0) break;
                out.write(w,0,size);
            }
            out.close();

            //HTTP接続のクローズ
            in.close();
            c.disconnect();
            return out.toByteArray();
        } catch (Exception e) {
            try {
                if (c!=null) c.disconnect();
                if (in!=null) in.close();
                if (out!=null) out.close();
            } catch (Exception e2) {
            }
            throw e;
        }
    }
    
    //トーストの表示
    private void toast(final String str) {
        handler.post(new Runnable(){public void run() {
            Toast.makeText(TwitterClient.this,str,Toast.LENGTH_LONG).show();
        }});
    }
    
    //Twitterアダプタの生成
    public class TwitterAdapter extends BaseAdapter {
        //項目数の取得
        @Override
        public int getCount() {
            return timeline.size();
        }

        //項目の取得
        @Override
        public Object getItem(int pos) {
            return timeline.get(pos);
        }

        //項目IDの取得
        @Override
        public long getItemId(int pos) {
            return pos;
        }

        //ビューの取得
        @Override
        public View getView(int pos,
            View convertView,ViewGroup parent) {
            Context context=TwitterClient.this;
            Status status=timeline.get(pos);
                
            //レイアウトの生成
            if (convertView==null) {
                int padding=(int)(6*dpi);
                LinearLayout layout=new LinearLayout(context);
                layout.setPadding(padding,padding,padding,padding);
                layout.setGravity(Gravity.TOP);
                convertView=layout;

                //アイコン
                int size=(int)(48*dpi);
                ImageView imageView=new ImageView(context);
                imageView.setTag("icon");
                imageView.setLayoutParams(new LinearLayout.LayoutParams(size,size));
                layout.addView(imageView);
                
                //テキストの指定
                TextView textView=new TextView(context);
                textView.setTag("text");
                textView.setPadding(padding,0,padding,0);
                layout.addView(textView);
            }
                
            //値の指定
            ImageView imageView=(ImageView)
                convertView.findViewWithTag("icon");
            imageView.setImageDrawable(status.icon);
            TextView textView=(TextView)
                convertView.findViewWithTag("text");
            textView.setText("【"+status.name+"】"+BR+status.text);
            return convertView;
        }        
    }
}